{***************************************************************
 *
 * Unit Name : Replication
 * Purpose   : Scala Replication
 * Author    : Eugen Mihailescu
 * Company   : EM Quicksoft Romania SRL
 * Copyright : 1998,2002  All rights reserved.
 * Web Site  : http://www.em-quicksoft.com
 * Support   : support@em-quicksoft.com
 * History   : 03/19/2000
 *
 * Disclaimer: The objectives of this program is only educational.
 *             You MUST NOT use it without getting any material benefit on it
 *             You have the right to share this code with others without removing
 *             the copyright and/or disclaimer notice.
 *             You are allowed to include its code in your application
 *             only if it was designed for non commercial purpose,
 *             otherwise we claim $10 for each copy of your
 *             application which contains this code inside.
 ****************************************************************}

Unit Replication;

Interface

Uses
  Db,
  Btr,
  btrtypes,
  BtrLib,
  DBTables,
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs;

Type
  TDoingWhat = (doGetModification, doCreateDBFile);
  TransferType = (ttUpdate, ttUpdateAppend, ttAppend);
  ModifyType = (mtNone, mtNew, mtModified, mtDeleted);
  TOnProgress = Procedure(Sender: TObject; DoingWhat: TDoingWhat; PercCompleted: Integer) Of Object;
  TFTPAddress = Class(TComponent)
  private
    fAddress: String;
    fUserName: String;
    fPassword: String;
    fPort: Integer;
    fDirectory: String;
    fisProxy: Boolean;
    fTimeOut: Integer;
  published
    Property Address: String read fAddress write fAddress;
    Property UserName: String read fUserName write fUserName;
    Property Password: String read fPassword write fPassword;
    Property Port: Integer read fPort write fPort;
    Property Directory: String read fDirectory write fDirectory;
    Property isProxy: Boolean read fisProxy write fisProxy;
    Property TimeOut: Integer read fTimeOut write fTimeOut;
  End;

  TMAILAddress = Class(TComponent)
  private
    fMailServer: String;
    fPort: Integer;
    fID: String;
    fMailFrom: String;
    fMailTo: String;
    fSubject: String;
    FLines: String;
    fisDialUp: Boolean;
    fisVolume: Boolean;
    fVolumeSize: Integer;
  published
    Property MailServer: String read fMailServer write fMailServer;
    Property Port: Integer read fPort write fPort;
    Property ID: String read fID write fID;
    Property MailFrom: String read fMailFrom write fMailFrom;
    Property MailTo: String read fMailTo write fMailTo;
    Property Subject: String read fSubject write fSubject;
    Property Lines: String read FLines write FLines;
    Property isDialUp: Boolean read fisDialUp write fisDialUp;
    Property isVolume: Boolean read fisVolume write fisVolume;
    Property VolumeSize: Integer read fVolumeSize write fVolumeSize;
  End;

  TSL01 = Record
    SL01018: Real;
    SL01038: Real;
    SL01046: Integer;
    SL01050: TDateTime;
    SL01051: TDateTime;
  End;

  TSC01 = Record
    SC01042: Integer;
    SC01043: Integer;
    SC01050: TDateTime;
    SC01051: TDateTime;
    SC01052: Real;
    SC01108: Real;
    SC01109: Real;
    SC01110: Real;
    SC01111: Real;
    SC01112: Integer;
    SC01113: Integer;
    SC01149: Real;
  End;

  TSC03 = Record
    SC03003: Integer;
    SC03004: Integer;
    SC03005: Integer;
    SC03006: Integer;
    SC03007: Integer;
    SC03008: Integer;
    SC03009: TDateTime;
    SC03011: Integer;
    SC03015: Integer;
    SC03016: Integer;
    SC03017: Integer;
    SC03025: Real;
    SC03029: Real;
    SC03054: TDateTime;
    SC03055: TDateTime;
    SC03056: TDateTime;
    SC03057: Real;
    SC03058: Real;
    SC03059: Real;
    SC03062: Integer;
    SC03063: Real;
    SC03064: Real;
    SC03065: Real;
    SC03066: Real;
    SC03067: Real;
    SC03082: Integer;
    SC03099: Real;
    SC03103: Integer;
  End;

  TSC07 = Record
    SC07022: String;
  End;

  TSC33 = Record
    SC33005: Integer;
    SC33006: Integer;
    SC33007: Integer;
    SC33008: Integer;
    SC33013: TDateTime;
    SC33023: Real;
  End;

  TSC06 = Record
    SC06004: Real;
  End;

  TReplication = Class(TComponent)
  private
    fAbort: Boolean;
    fAbortMsg: String;
    FOnProgress: TOnProgress;
  protected
  public
    Procedure TransferFile(DestPath: String; FileName: String; NewFile: String; TType:
      TransferType; Index: Integer; TranzTypes: String);
    Procedure TransferUpdate(DestPath: String; SourceFileName: String; ParadoxNewFile: String;
      Index: Integer; TranzTypes: String);
    Procedure CreateAlias(AliasName: String; AliasPath: String);
    Function CreateBtrDBFile(FileName: String; NewFile: String; IsEmpty: Boolean; TranzTypes: String): Boolean;
    Procedure GetModification(OldFile, NewFile, ModfFile: String);
    Function CONSOLIDATE_SL01(VALUE1, VALUE2: TSL01): TSL01;
    Function CONSOLIDATE_SC01(VALUE1, VALUE2: TSC01): TSC01;
    Function CONSOLIDATE_SC03(VALUE1, VALUE2: TSC03): TSC03;
    Function CONSOLIDATE_SC33(VALUE1, VALUE2: TSC33): TSC33;
    Function CONSOLIDATE_SC06(VALUE1, VALUE2: TSC06): TSC06;
    Function GET_SC07_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
      String;
    Function GET_SL04_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
      String;
    Function GET_SL05_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
      String;
    Function GET_SL21_NEXTINDEX(BTRFILENAME, SL21001, SL21002, SL21003: String;
      POZ1, LEN1, POZ2, LEN2, POZ3, LEN3: Integer): String;
    Function GET_PL04_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
      String;
    Function GET_PL05_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
      String;
    Function GET_PL21_NEXTINDEX(BTRFILENAME, PL21001, PL21002, PL21003: String;
      POZ1, LEN1, POZ2, LEN2, POZ3, LEN3: Integer): String;
    Function GetBtrRecCount(FileName: String): Longint;
  published
    Property AbortMsg: String read fAbortMsg write fAbortMsg;
    Property abort: Boolean read fAbort write fAbort;
    Property OnProgress: TOnProgress read FOnProgress write FOnProgress;
  End;

Var
  MaxProgress, Progress: Integer;

Function StringToComponent(Value: String): TComponent;
Function ComponentToString(Component: TComponent): String;

Procedure Register;

Implementation
//******************* ComponentToString *************************

Function ComponentToString(Component: TComponent): String;

Var
  BinStream         : TMemoryStream;
  StrStream         : TStringStream;
  S                 : String;
Begin
  BinStream := TMemoryStream.Create;
  Try
    StrStream := TStringStream.Create(S);
    Try
      BinStream.WriteComponent(Component);
      BinStream.Seek(0, soFromBeginning);
      ObjectBinaryToText(BinStream, StrStream);
      StrStream.Seek(0, soFromBeginning);
      Result := StrStream.DataString;
    Finally
      StrStream.Free;
    End;

  Finally
    BinStream.Free
  End;
End;
//******************* StringToComponent *************************

Function StringToComponent(Value: String): TComponent;
Var
  StrStream         : TStringStream;
  BinStream         : TMemoryStream;
Begin
  StrStream := TStringStream.Create(Value);
  Try
    BinStream := TMemoryStream.Create;
    Try
      ObjectTextToBinary(StrStream, BinStream);
      BinStream.Seek(0, soFromBeginning);
      Result := BinStream.ReadComponent(Nil);

    Finally
      BinStream.Free;
    End;
  Finally
    StrStream.Free;
  End;
End;
//******************* TReplication.GetModification *************************

Procedure TReplication.GetModification(OldFile, NewFile, ModfFile: String);
Var
  T1, T2, T3        : TTable;
  I                 : Integer;
  DS                : TDataSource;
  MT                : ModifyType;
Begin
  If Assigned(FOnProgress) Then
    FOnProgress(Self, doGetModification, 0);

  T1 := TTable.Create(Application);
  T2 := TTable.Create(Application);
  T3 := TTable.Create(Application);
  DS := TDataSource.Create(Application);

  DS.DataSet := T2;
  T1.TableType := ttParadox;
  T2.TableType := ttParadox;
  T3.TableType := ttParadox;
  T1.TableName := OldFile;
  T2.TableName := NewFile;
  T3.TableName := ModfFile;
  T1.MasterSource := DS;
  T1.MasterFields := '';
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC01' Then
    Begin
      T1.MasterFields := 'SC01001';
      T2.IndexFieldNames := 'SC01001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC03' Then
    Begin
      T1.MasterFields := 'SC03001';
      T2.IndexFieldNames := 'SC03001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC07' Then
    Begin
      T1.MasterFields := 'SC07022';
      T2.IndexFieldNames := 'SC07022';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC12' Then
    Begin
      T1.MasterFields := 'SC12001';
      T2.IndexFieldNames := 'SC12001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC33' Then
    Begin
      T1.MasterFields := 'SC33001';
      T2.IndexFieldNames := 'SC33001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL01' Then
    Begin
      T1.MasterFields := 'SL01001';
      T2.IndexFieldNames := 'SL01001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL03' Then
    Begin
      T1.MasterFields := 'SL03001';
      T2.IndexFieldNames := 'SL03001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL04' Then
    Begin
      T1.MasterFields := 'SL04001';
      T2.IndexFieldNames := 'SL04001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL21' Then
    Begin
      T1.MasterFields := 'SL21001';
      T2.IndexFieldNames := 'SL21001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL05' Then
    Begin
      T1.MasterFields := 'SL05001';
      T2.IndexFieldNames := 'SL05001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'PL03' Then
    Begin
      T1.MasterFields := 'PL03001';
      T2.IndexFieldNames := 'PL03001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'PL04' Then
    Begin
      T1.MasterFields := 'PL04001';
      T2.IndexFieldNames := 'PL04001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'PL21' Then
    Begin
      T1.MasterFields := 'PL21001';
      T2.IndexFieldNames := 'PL21001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'PL05' Then
    Begin
      T1.MasterFields := 'PL05001';
      T2.IndexFieldNames := 'PL05001';
    End;

  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC06' Then
    Begin
      T1.MasterFields := 'SC06001';
      T2.IndexFieldNames := 'SC06001';
    End;

  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'ST01' Then
    Begin
      T1.MasterFields := 'ST01001';
      T2.IndexFieldNames := 'ST01001';
    End;

  T1.Active := True;
  T2.Active := True;
  T3.FieldDefs.Assign(T1.FieldDefs);
  T3.FieldDefs.Add('StatusID', ftInteger, 0, False);
  T3.CreateTable;
  T3.IndexDefs.Clear;
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SC01' Then
    T3.AddIndex('SC01001', 'SC01001', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SC03' Then
    T3.AddIndex('SC03001', 'SC03001;SC03002', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SC07' Then
    T3.AddIndex('SC07022', 'SC07022', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SL01' Then
    T3.AddIndex('SL01001', 'SL01001', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SL03' Then
    T3.AddIndex('SL03001', 'SL03001;SL03002', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SL04' Then
    T3.AddIndex('SL04001', 'SL04001;SL04002', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SL21' Then
    T3.AddIndex('SL21001', 'SL21001;SL21002;SL21003', [ixPrimary, ixUnique]
      );
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SL05' Then
    T3.AddIndex('SL05001', 'SL05001;SL05002', [ixPrimary, ixUnique]);

  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'PL03' Then
    T3.AddIndex('PL03001', 'PL03001;PL03002', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'PL04' Then
    T3.AddIndex('PL04001', 'PL04001;PL04002', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'PL21' Then
    T3.AddIndex('PL21001', 'PL21001;PL21002;PL21003', [ixPrimary, ixUnique]
      );
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'PL05' Then
    T3.AddIndex('PL05001', 'PL05001;PL05002', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'SC06' Then
    T3.AddIndex('SC06001', 'SC06001;SC06002;SC06003', [ixPrimary, ixUnique]);
  If UpperCase(Copy(ExtractFileName(T3.TableName), 1, 4)) = 'ST01' Then
    T3.AddIndex('ST01001', 'ST01001', [ixPrimary, ixUnique]);

  T3.Active := True;
  T2.First;
  Progress := 0;
  T1.MasterSource := Nil;
  maxprogress := T2.RecordCount + T1.RecordCount;
  T1.MasterSource := DS;
  While Not T2.Eof Do
    Begin
      Inc(Progress);
      If (Assigned(FOnProgress)) And (MaxProgress <> 0) Then
        FOnProgress(Self, doGetModification, Trunc(100 * Progress / MaxProgress));
      Application.ProcessMessages;
      MT := mtNone;
      If T1.RecordCount = 0 Then
        MT := mtNew
      Else
        For I := 0 To T1.FieldCount - 1 Do
          If T1.Fields[I].AsVariant <> T2.Fields[I].AsVariant Then
            Begin
              MT := mtModified;
              Break;
            End;

      If MT <> mtNone Then
        Begin
          T3.Append;
          T3.Edit;
          For I := 0 To T2.FieldCount - 1 Do
            Begin
              T3.Fields[I].AsVariant := T2.Fields[I].AsVariant;
              If MT = mtModified Then
                Begin
                  If T2.FieldDefs.Items[I].Name = 'SL01038' Then
                    T3.Fields[I].asFloat := T2.Fields[I].asFloat -
                      T1.Fields[I].asFloat;
                  If T2.FieldDefs.Items[I].Name = 'SC01042' Then
                    T3.Fields[I].asFloat := T2.Fields[I].asFloat -
                      T1.Fields[I].asFloat;
                  If T2.FieldDefs.Items[I].Name = 'SC03003' Then
                    T3.Fields[I].asFloat := T2.Fields[I].asFloat -
                      T1.Fields[I].asFloat;
                  If T2.FieldDefs.Items[I].Name = 'SC06004' Then
                    T3.Fields[I].asFloat := T2.Fields[I].asFloat -
                      T1.Fields[I].asFloat;
                End;
            End;
          T3.FieldByName('StatusID').AsInteger := Ord(MT);
          T3.Post;
        End;
      T2.Next;
    End;
  T1.Active := False;
  T2.Active := False;

  T1.MasterSource := Nil;
  T1.MasterFields := '';
  DS.DataSet := T1;
  T2.MasterSource := DS;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC01' Then
    Begin
      T2.MasterFields := 'SC01001';
      T1.IndexFieldNames := 'SC01001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC03' Then
    Begin
      T2.MasterFields := 'SC03001';
      T1.IndexFieldNames := 'SC03001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC07' Then
    Begin
      T2.MasterFields := '';
      T1.IndexFieldNames := '';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC12' Then
    Begin
      T2.MasterFields := 'SC12001';
      T1.IndexFieldNames := 'SC12001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC33' Then
    Begin
      T2.MasterFields := 'SC33001';
      T1.IndexFieldNames := 'SC33001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL01' Then
    Begin
      T2.MasterFields := 'SL01001';
      T1.IndexFieldNames := 'SL01001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL03' Then
    Begin
      T2.MasterFields := 'SL03001';
      T1.IndexFieldNames := 'SL03001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL04' Then
    Begin
      T2.MasterFields := 'SL04001';
      T1.IndexFieldNames := 'SL04001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL05' Then
    Begin
      T2.MasterFields := 'SL05001';
      T1.IndexFieldNames := 'SL05001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SL21' Then
    Begin
      T2.MasterFields := 'SL21001';
      T1.IndexFieldNames := 'SL21001';
    End;

  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'PL03' Then
    Begin
      T2.MasterFields := 'PL03001';
      T1.IndexFieldNames := 'PL03001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'PL04' Then
    Begin
      T2.MasterFields := 'PL04001';
      T1.IndexFieldNames := 'PL04001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'PL05' Then
    Begin
      T2.MasterFields := 'PL05001';
      T1.IndexFieldNames := 'PL05001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'PL21' Then
    Begin
      T2.MasterFields := 'PL21001';
      T1.IndexFieldNames := 'PL21001';
    End;
  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'SC06' Then
    Begin
      T2.MasterFields := 'SC06001';
      T1.IndexFieldNames := 'SC06001';
    End;

  If UpperCase(Copy(ExtractFileName(T2.TableName), 1, 4)) = 'ST01' Then
    Begin
      T2.MasterFields := 'ST01001';
      T1.IndexFieldNames := 'ST01001';
    End;

  T1.Active := True;
  T2.Active := True;
  T2.First;
  While Not T1.Eof Do
    Begin
      Inc(Progress);
      If (Assigned(FOnProgress)) And (MaxProgress <> 0) Then
        FOnProgress(Self, doGetModification, Trunc(100 * Progress / MaxProgress));
      MT := mtNone;
      If T2.RecordCount = 0 Then
        MT := mtDeleted;
      If MT <> mtNone Then
        Begin
          T3.Append;
          T3.Edit;
          For I := 0 To T2.FieldCount - 1 Do
            Begin
              T3.Fields[I].AsVariant := T1.Fields[I].AsVariant;
              If MT = mtModified Then
                Begin
                  If T2.FieldDefs.Items[I].Name = 'SL01038' Then
                    T3.Fields[I].asFloat := T2.Fields[I].asFloat -
                      T1.Fields[I].asFloat;
                  If T2.FieldDefs.Items[I].Name = 'SC01042' Then
                    T3.Fields[I].asFloat := T2.Fields[I].asFloat -
                      T1.Fields[I].asFloat;
                  If T2.FieldDefs.Items[I].Name = 'SC03003' Then
                    T3.Fields[I].asFloat := T2.Fields[I].asFloat -
                      T1.Fields[I].asFloat;
                  If T2.FieldDefs.Items[I].Name = 'SC06004' Then
                    T3.Fields[I].asFloat := T2.Fields[I].asFloat -
                      T1.Fields[I].asFloat;
                End;
            End;
          T3.FieldByName('StatusID').AsInteger := Ord(MT);
          T3.Post;
        End;
      T1.Next;
    End;
  T1.Active := False;
  T2.Active := False;

  T3.Active := False;
End;
//******************* TReplication.CreateBtrDBFile *************************

Function TReplication.CreateBtrDBFile(FileName: String; NewFile: String; IsEmpty: Boolean; TranzTypes: String): Boolean;
Var
  T                 : TTable;
  ET                : TTable;
  FldType           : TFieldType;
  Size              : Integer;
  BtrFile           : TBtr;
  I, j, k           : Integer;
  p                 : Pointer;
  pozlen            : Array[1..999, 1..2] Of Integer;
  info              : Array[1..999] Of Variant;
  askagain          : Boolean;
  TT                : TStringList;
  IgnoreAll         : Boolean;
Begin
  If Assigned(FOnProgress) Then
    FOnProgress(Self, doCreateDBFile, 0);
  Result := False;
  TT := TStringList.Create;
  Try
    TT.CommaText := TranzTypes;
    T := TTable.Create(Application);
    ET := TTable.Create(Application);
    BtrFile := TBtr.Create(Application);
    BtrFile.FileName := FileName;
    BtrFile.IgnoreSmall := True;
    BtrFile.FirstOnOpen := True;
    Try
      BtrFile.Active := True;
    Except
      On E: Exception Do
        Begin
          mymessagedlg('Eroare', PChar('Fisierul ' + ExtractFileName(FileName
            ) +
            ' nu poate fi deschis.'#13#10'Mesaj sistem: ' + E.Message),
            mtInformation, [mbOK]);
          Result := False;
          Exit;
        End;
    End;
    T.TableType := ttParadox;
    ET.TableType := ttParadox;
    T.SessionName := 'Default';
    ET.SessionName := 'Default';
    T.DatabaseName := 'ScalaDef';
    ET.DatabaseName := 'ScaTransfer';
    T.TableName := 'DBField';
    T.IndexFieldNames := 'FILENO;FIELDCODE';
    ET.TableName := NewFile;

    ET.FieldDefs.Clear;
    T.Filter := 'FILENO=''' + Copy(ExtractFileName(FileName), 1, 4) + '''';
    T.Filtered := True;
    j := 2;
    Try
      T.Active := True;
      T.First;
      Progress := 0;
      Maxprogress := T.RecordCount + BtrFile.RecordCount;
      If Copy(ExtractFileName(FileName), 1, 4) <> 'SC07' Then
        While Not T.Eof Do
          Begin
            Inc(Progress);
            Application.ProcessMessages;
            Case T.FieldByName('DataType').AsInteger Of
              0, 1, 6, 9, 10, 13:
                Begin
                  FldType := ftString;
                  Size := T.FieldByName('SIZE').AsInteger;
                End;
              2, 3, 7, 8, 12:
                Begin
                  FldType := ftFloat;
                  Size := 0;
                End;
              11:
                Begin
                  FldType := ftBoolean;
                  Size := 0;
                End;
              5:
                Begin
                  FldType := ftDate;
                  Size := 0;
                End;
              4:
                Begin
                  FldType := ftInteger;
                  Size := 0;
                End;
              Else
                Begin
                  FldType := ftString;
                  Size := T.FieldByName('SIZE').AsInteger;
                End;
            End;
            pozlen[j - 1, 1] := T.FieldByName('OFFSET').AsInteger + 1;
            pozlen[j - 1, 2] := T.FieldByName('SIZE').AsInteger;
            ET.FieldDefs.Add(T.FieldByName('FIELDCODE').AsString, FldType,
              Size, False);
            T.Next;
            If Not T.Eof Then Inc(j);
          End
      Else                              //FILENAME=SC07
        Begin
          ET.FieldDefs.Add('SC07022', ftString, 8, False);
          pozlen[1, 1] := 233;
          pozlen[1, 2] := 8;
          j := 2;
          While Not T.Eof Do
            Begin
              Inc(Progress);
              Application.ProcessMessages;
              Case T.FieldByName('DataType').AsInteger Of
                0, 1, 6, 9, 10, 13:
                  Begin
                    FldType := ftString;
                    Size := T.FieldByName('SIZE').AsInteger;
                  End;
                2, 3, 7, 8, 12:
                  Begin
                    FldType := ftFloat;
                    Size := 0;
                  End;
                11:
                  Begin
                    FldType := ftBoolean;
                    Size := 0;
                  End;
                5:
                  Begin
                    FldType := ftDate;
                    Size := 0;
                  End;
                4:
                  Begin
                    FldType := ftInteger;
                    Size := 0;
                  End;
                Else
                  Begin
                    FldType := ftString;
                    Size := T.FieldByName('SIZE').AsInteger;
                  End;
              End;
              If T.FieldByName('FIELDCODE').AsString <> 'SC07022' Then
                Begin
                  pozlen[j, 1] := T.FieldByName('OFFSET').AsInteger + 1;
                  pozlen[j, 2] := T.FieldByName('SIZE').AsInteger;
                  ET.FieldDefs.Add(T.FieldByName('FIELDCODE').AsString,
                    FldType, Size, False);
                End;
              If (Not T.Eof) And (T.FieldByName('FIELDCODE').AsString <>
                'SC07022') Then Inc(j);
              T.Next;
            End
        End;
      T.Active := False;
      If ET.FieldDefs.Count > 0 Then
        ET.CreateTable
      Else
        Begin
          mymessagedlg('Atentie', PChar('Fisierul ' + ExtractFileName(
            FileName) +
            ' nu are definitia in DBFIELD.DB. Fisierul nu poate fi creat.')
            , mtInformation, [mbOK]);
          Result := False;
          Exit;
        End;
      ET.IndexDefs.Clear;
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SC01' Then
        ET.AddIndex('SC01001', 'SC01001', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SC03' Then
        ET.AddIndex('SC03001', 'SC03001;SC03002', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SC07' Then ET.AddIndex('SC07022', 'SC07022', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SC33' Then
        ET.AddIndex('SC33001', 'SC33001;SC33002;SC33003', [ixPrimary, ixUnique]
          );
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SL01' Then
        ET.AddIndex('SL01001', 'SL01001', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'GL07' Then
        ET.AddIndex('GL07001', 'GL07001;GL07002', [ixPrimary, ixUnique]);

      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SL03' Then
        ET.AddIndex('SL03001', 'SL03001;SL03002', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SL04' Then
        ET.AddIndex('SL04001', 'SL04001;SL04002', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SL05' Then
        ET.AddIndex('SL05001', 'SL05001;SL05002', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SL21' Then
        ET.AddIndex('SL21001', 'SL21001;SL21002;SL21003', [ixPrimary, ixUnique]
          );

      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'PL03' Then
        ET.AddIndex('PL03001', 'PL03001;PL03002', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'PL04' Then
        ET.AddIndex('PL04001', 'PL04001;PL04002', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'PL05' Then
        ET.AddIndex('PL05001', 'PL05001;PL05002', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'PL21' Then
        ET.AddIndex('PL21001', 'PL21001;PL21002;PL21003', [ixPrimary, ixUnique]
          );
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SC06' Then
        ET.AddIndex('SC06001', 'SC06001;SC06002;SC06003', [ixPrimary, ixUnique]);

      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'ST01' Then
        ET.AddIndex('ST01001', 'ST01001', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'PA04' Then
        ET.AddIndex('PA04001', 'PA04001', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'PA03' Then
        ET.AddIndex('PA03001', 'PA03001', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'PA06' Then
        ET.AddIndex('PA06001', 'PA06001;PA06002;PA06003;PA06004', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'PA39' Then
        ET.AddIndex('PA39001', 'PA39001;PA39002;PA39003', [ixPrimary, ixUnique]);
      If UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SY24' Then
        ET.AddIndex('SY24001', 'SY24001;SY24002', [ixPrimary]);

      If IsEmpty Then
        Begin
          Result := True;
          Exit;
        End;
      ET.Active := True;
      askagain := True;
      IgnoreAll := False;
      If Not fAbort Then
        For I := 1 To BtrFile.RecordCount Do
          Begin
            Try
              If (fAbort) And (MessageDlg(fAbortMsg, mtConfirmation, [mbYes, mbNo], 0) = mrYes) Then
                Break
              Else
                fAbort := False;
              p := BtrFile.DataBuffer;
              If (UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) <> 'SC07') Or (((UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SC07') And (TT.IndexOf(Copy(String(p), 1, 2)) >= 0)) Or ((UpperCase(Copy(ExtractFileName(ET.TableName), 1, 4)) = 'SC07') And (TT.Count = 0))) Then
                Begin
                  ET.Append;
                  Inc(Progress);
                  If (Assigned(FOnProgress)) And (MaxProgress <> 0) Then
                    FOnProgress(Self, doCreateDBFile, Trunc(100 * Progress / MaxProgress));
                  Application.ProcessMessages;
                  ET.Edit;
                  For k := 1 To j - 1 Do
                    Begin
                      info[k] := trimleft(trimright(Copy(String(p), pozlen[k, 1], pozlen[k, 2])));

                      Try
                        If ET.FieldDefs.Items[k - 1].DataType In [ftSmallInt, ftWord, ftInteger] Then
                          info[k] := IntAsSystem(info[k]);
                      Except
                        If askagain Then
                          If myMessageDlg('Atentie', PChar(
                            'O eroare a aparut cand s-a incercat conversia Paradox-Btrieve a valorii de tip INTEGER: ' + String
                            (info
                            [k]) + '. Vezi linia: ' + IntToStr(I) + ', campul: ' +
                            IntToStr(k) + ' din fisierul ' + btrfile.FileName +
                            ' pentru detalii suplimentare.'), mtWarning,
                            [mbOK, mbAll]) = 8 Then askagain := False;
                        info[k] := trimleft(trimright(Copy(String(p), pozlen[
                          k,
                            1], pozlen[k, 2])));
                      End;

                      Try
                        If (ET.FieldDefs.Items[k - 1].DataType In [ftFloat, ftCurrency
                          ]) Then
                          info[k] := FloatAsSystem(info[k]);
                      Except
                        If askagain Then
                          If myMessageDlg('Atentie', PChar(
                            'O eroare a aparut cand s-a incercat conversia Paradox-Btrieve a valorii de tip REAL : ' + String
                            (info
                            [k]) + '. Vezi linia: ' + IntToStr(I) + ', campul: ' +
                            IntToStr
                            (k) + ' din fisierul ' + btrfile.FileName +
                            ' pentru detalii suplimentare.'), mtWarning,
                            [mbOK, mbAll]) = 8 Then askagain := False;
                        info[k] := trimleft(trimright(Copy(String(p), pozlen[
                          k,
                            1], pozlen[k, 2])));
                      End;

                      Try
                        If ET.FieldDefs.Items[k - 1].DataType In [ftDateTime, ftDate] Then
                          info[k] := DateAsSystem(info[k], 1);
                      Except
                        If askagain Then
                          If myMessageDlg('Atentie', PChar(
                            'O eroare a aparut cand s-a incercat conversia Paradox-Btrieve a valorii de tip DATE: ' + String
                            (info
                            [k]) + '. Vezi linia: ' + IntToStr(I) + ', campul: ' +
                            IntToStr
                            (k) + ' din fisierul ' + btrfile.FileName +
                            ' pentru detalii suplimentare.'), mtWarning,
                            [mbOK, mbAll]) = 8 Then askagain := False;
                        info[k] := trimleft(trimright(Copy(String(p), pozlen[
                          k,
                            1], pozlen[k, 2])));
                      End;

                      Try
                        If ET.FieldDefs.Items[k - 1].DataType = ftBoolean Then
                          If info[k] = '1' Then
                            info[k] := True
                          Else
                            info[k] := False;
                      Except
                        If askagain Then
                          If myMessageDlg('Atentie', PChar(
                            'O eroare a aparut cand s-a incercat conversia Paradox-Btrieve a valorii de tip BOOLEAN: ' + String
                            (info
                            [k]) + '. Vezi linia: ' + IntToStr(I) + ', campul: ' +
                            IntToStr
                            (k) + ' din fisierul ' + btrfile.FileName +
                            ' pentru detalii suplimentare.'), mtWarning,
                            [mbOK, mbAll]) = 8 Then askagain := False;
                        info[k] := trimleft(trimright(Copy(String(p), pozlen[
                          k,
                            1], pozlen[k, 2])));
                      End;

                      Try
                        ET.Fields[k - 1].AsVariant := info[k];
                      Except
                      End;

                    End;
                  ET.Post;
                End;
              Try
                BtrFile.Next;
              Except
              End;
            Except
              On E: Exception Do
                If Not IgnoreAll Then
                  IgnoreAll := myMessageDlg('Eroare',
                    'O eroare a aparut la citirea dictionarului DBFIELD.DB. Operatiune ignorata.'#13#10'Mesaj de sistem: '
                    + E.Message, mtInformation, [mbIgnore, mbAll]) = mrAll;
            End;
          End;

      Try
        BtrFile.Active := False;
        ET.Active := False;
      Except
      End;
    Except
      On E: Exception Do
        Begin
          myMessageDlg('Eroare',
            'O eroare a aparut la citirea dictionarului DBFIELD.DB. Operatiune ignorata.'#13#10'Mesaj de sistem: '
            + E.Message, mtInformation, [mbOK]);
          Result := False;
          Exit;
        End;
    End;
  Finally
    TT.Free;
  End;
End;
//******************* TReplication.CreateAlias *************************

Procedure TReplication.CreateAlias(AliasName: String; AliasPath: String);
Begin
  If Not Session.Active Then
    Session.Active := True;
  If Not Session.IsAlias('ScaTransfer') Then
    Begin
      Try
        Session.AddStandardAlias('ScaTransfer',
          ExtractFilePath(Application.ExeName), 'PARADOX');
      Except
        On E: Exception Do
          myMessageDlg('Eroare', PChar(E.Message), mtInformation, [mbOK]
            );
      End;
    End;
End;
//******************* TReplication.TransferFile *************************

Procedure TReplication.TransferFile(DestPath: String; FileName: String; NewFile: String; TType:
  TransferType; Index: Integer; TranzTypes: String);
Begin
  If TType = ttUpdate Then
    TransferUpdate(DestPath, FileName, NewFile, Index, TranzTypes);
End;
//******************* TReplication.TransferUpdate *************************

Procedure TReplication.TransferUpdate(DestPath: String; SourceFileName: String; ParadoxNewFile: String;
  Index: Integer; TranzTypes: String);
Begin
  CreateAlias('ScaTransfer', ExtractFilePath(Application.ExeName));
  If Index = 1 Then
    If Not fileexists(DestPath +
      Copy(ExtractFileName(SourceFileName), 1, Length(ExtractFileName(
      SourceFileName)
      ) - 4) + '_' + FormatFloat('0000', Index) + '.DB') Then
      Begin
        CreateBtrDBFile(SourceFileName, ParadoxNewFile, True, TranzTypes);
        CreateBtrDBFile(SourceFileName, DestPath +
          Copy(ExtractFileName(SourceFileName), 1, 4) + 'New_' +
          FormatFloat('0000', Index) + '.DB', False, TranzTypes)
      End
    Else
      CreateBtrDBFile(SourceFileName, DestPath
        +
        Copy(ExtractFileName(SourceFileName), 1, 4) + 'New_' + FormatFloat
        ('0000', Index) + '.DB', False, TranzTypes);

  If Index > 1 Then
    If Not fileexists(DestPath +
      Copy(ExtractFileName(SourceFileName), 1, Length(ExtractFileName(
      SourceFileName)
      ) - 4) + '_' + FormatFloat('0000', Index) + '.DB') Then
      Begin
        myMessageDlg('Fisier negasit', 'Fisierul ' + Copy(ExtractFileName(
          SourceFileName), 1, Length(ExtractFileName(SourceFileName)
          ) - 4) + '_' + FormatFloat('0000', Index) + '.DB' +
          ' lipseste de pe disc. Implicit se va crea un nou fisier in care toate inregistrarile vor fi marcate ca inregistrari noi.',
          mtWarning, [mbOK]);
        CreateBtrDBFile(SourceFileName, ParadoxNewFile, True, TranzTypes);
        CreateBtrDBFile(SourceFileName, DestPath
          +
          Copy(ExtractFileName(SourceFileName), 1, 4) + 'New_' +
          FormatFloat('0000', Index) + '.DB', False, TranzTypes)
      End
    Else
      CreateBtrDBFile(SourceFileName, DestPath
        +
        Copy(ExtractFileName(SourceFileName), 1, 4) + 'New_' + FormatFloat
        ('0000', Index) + '.DB', False, TranzTypes);

  GetModification(DestPath +
    Copy(ExtractFileName(SourceFileName), 1, Length(ExtractFileName(
    SourceFileName)
    ) - 4) + '_' + FormatFloat('0000', Index) + '.DB', DestPath +
    Copy(ExtractFileName(SourceFileName), 1, 4) + 'New_' + FormatFloat(
    '0000', Index) + '.DB',
    DestPath + Copy(ExtractFileName(
    SourceFileName), 1, 4) + 'Modf_' + FormatFloat('0000', Index) + '.DB'
    );
End;
//******************* TReplication.CONSOLIDATE_SL01 *************************

Function TReplication.CONSOLIDATE_SL01(VALUE1, VALUE2: TSL01): TSL01;
Var
  R                 : TSL01;
Begin
  R.SL01018 := VALUE1.SL01018 + VALUE2.SL01018;
  R.SL01038 := VALUE1.SL01038 + VALUE2.SL01038;
  R.SL01046 := VALUE1.SL01046 + VALUE2.SL01046;
  If VALUE1.SL01050 > VALUE2.SL01050 Then
    R.SL01050 := VALUE1.SL01050
  Else
    R.SL01050 := VALUE2.SL01050;
  If VALUE1.SL01051 > VALUE2.SL01051 Then
    R.SL01051 := VALUE1.SL01051
  Else
    R.SL01051 := VALUE2.SL01051;
  Result := R;
End;
//******************* TReplication.CONSOLIDATE_SC01 *************************

Function TReplication.CONSOLIDATE_SC01(VALUE1, VALUE2: TSC01): TSC01;
Var
  R                 : TSC01;
Begin
  R.SC01042 := VALUE1.SC01042 + VALUE2.SC01042;
  If VALUE1.SC01050 > VALUE2.SC01050 Then
    R.SC01050 := VALUE1.SC01050
  Else
    R.SC01050 := VALUE2.SC01050;
  If VALUE1.SC01051 > VALUE2.SC01051 Then
    R.SC01051 := VALUE1.SC01051
  Else
    R.SC01051 := VALUE2.SC01051;
  R.SC01108 := VALUE1.SC01108 + VALUE2.SC01108;
  R.SC01109 := VALUE1.SC01109 + VALUE2.SC01109;
  R.SC01110 := VALUE1.SC01110 + VALUE2.SC01110;
  R.SC01111 := VALUE1.SC01111 + VALUE2.SC01111;
  R.SC01112 := VALUE1.SC01112 + VALUE2.SC01112;
  R.SC01113 := VALUE1.SC01113 + VALUE2.SC01113;
  Result := R;
End;
//******************* TReplication.CONSOLIDATE_SC03 *************************

Function TReplication.CONSOLIDATE_SC03(VALUE1, VALUE2: TSC03): TSC03;
Var
  R                 : TSC03;
Begin
  R.SC03003 := VALUE1.SC03003 + VALUE2.SC03003;
  If VALUE1.SC03055 > VALUE2.SC03055 Then
    R.SC03055 := VALUE1.SC03055
  Else
    R.SC03055 := VALUE2.SC03055;
  Result := R;
End;
//******************* TReplication.CONSOLIDATE_SC06 *************************

Function TReplication.CONSOLIDATE_SC06(VALUE1, VALUE2: TSC06): TSC06;
Var
  R                 : TSC06;
Begin
  R.SC06004 := VALUE1.SC06004 + VALUE2.SC06004;
  Result := R;
End;
//******************* TReplication.GET_SC07_NEXTINDEX *************************

Function TReplication.GET_SC07_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
  String;
Var
  BTR_FILE          : TBTR;
  R                 : Integer;
  S                 : String;
Begin
  BTR_FILE := TBTR.Create(Application);
  Try
    BTR_FILE.FileName := BTRFILENAME;
    BTR_FILE.IgnoreSmall := True;
    Try
      BTR_FILE.Active := True;
      If BTR_FILE.RecordCount = 0 Then
        R := 0
      Else
        Begin
          BTR_FILE.Last;
          S := Copy(String(BTR_FILE.DATABUFFER), POZ + 1, LEN);
          R := STRTONR(S);
          Inc(R);
        End;
      BTR_FILE.Active := False;
    Except
      R := 0;
    End;
  Finally
    BTR_FILE.Free;
  End;
  Result := FormatFloat(StringOfChar('0', LEN), R);
End;
//******************* TReplication.GET_SL04_NEXTINDEX *************************

Function TReplication.GET_SL04_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
  String;
Var
  BTR_FILE          : TBTR;
  R                 : Integer;
  S                 : String;
Begin
  BTR_FILE := TBTR.Create(Application);
  Try
    BTR_FILE.FileName := BTRFILENAME;
    BTR_FILE.IgnoreSmall := True;
    Try
      BTR_FILE.Active := True;
      If BTR_FILE.RecordCount = 0 Then
        R := 0
      Else
        Begin
          BTR_FILE.First;
          S := Copy(String(BTR_FILE.DATABUFFER), POZ + 1, LEN);
          R := STRTONR(S);
          Inc(R);
        End;
      BTR_FILE.Active := False;
    Except
      R := 0;
    End;
  Finally
    BTR_FILE.Free;
  End;
  Result := FormatFloat(StringOfChar('0', LEN), R);
End;
//******************* TReplication.GET_SL05_NEXTINDEX *************************

Function TReplication.GET_SL05_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
  String;
Var
  BTR_FILE          : TBTR;
  R                 : Integer;
  S                 : String;
Begin
  BTR_FILE := TBTR.Create(Application);
  Try
    BTR_FILE.FileName := BTRFILENAME;
    BTR_FILE.IgnoreSmall := True;
    Try
      BTR_FILE.Active := True;
      If BTR_FILE.RecordCount = 0 Then
        R := 0
      Else
        Begin
          BTR_FILE.First;
          S := Copy(String(BTR_FILE.DATABUFFER), POZ + 1, LEN);
          R := STRTONR(S);
          Inc(R);
        End;
      BTR_FILE.Active := False;
    Except
      R := 0;
    End;
  Finally
    BTR_FILE.Free;
  End;
  Result := FormatFloat(StringOfChar('0', LEN), R);
End;
//******************* TReplication.GET_SL21_NEXTINDEX *************************

Function TReplication.GET_SL21_NEXTINDEX(BTRFILENAME, SL21001, SL21002, SL21003: String;
  POZ1, LEN1, POZ2, LEN2, POZ3, LEN3: Integer): String;
Var
  BTR_FILE          : TBTR;
  R                 : Integer;
  S                 : String;
  KEYB              : CHARARRAY;
Begin
  R := 1;
  BTR_FILE := TBTR.Create(Application);
  Try
    BTR_FILE.FileName := BTRFILENAME;
    BTR_FILE.IgnoreSmall := True;
    BTR_FILE.KeyNum := 0;
    Try
      BTR_FILE.Active := True;
      If BTR_FILE.RecordCount = 0 Then
        R := 1
      Else
        Begin
          BTR_FILE.First;
          StrPCopy(KEYB, Format('%-' + IntToStr(LEN1) + 's' + '%-' +
            IntToStr(LEN2) + 's' + '%-' + IntToStr(LEN3) + 's', [SL21001,
            SL21002, SL21003]));
          StrCopy(BTR_FILE.KeyBuffer, KEYB);
          Try
            BTR_FILE.Equal;
            While (trimright(Copy(String(BTR_FILE.DATABUFFER), POZ1 + 1,
              LEN1)) = SL21001) And (trimright(Copy(String(
              BTR_FILE.DATABUFFER), POZ2 + 1, LEN2)) = SL21002) Do
              Begin
                S := trimright(Copy(String(BTR_FILE.DATABUFFER), POZ3 +
                  1, LEN3));
                R := STRTONR(S);
                Try
                  BTR_FILE.Next;
                Except
                  Break;
                End;
              End;
            Inc(R);
          Except
            R := 1;
          End;
        End;
      BTR_FILE.Active := False;
    Except
      R := 1;
    End;
  Finally
    BTR_FILE.Free;
  End;
  Result := FormatFloat(StringOfChar('0', LEN3), R);
End;
//******************* TReplication.CONSOLIDATE_SC33 *************************

Function TReplication.CONSOLIDATE_SC33(VALUE1, VALUE2: TSC33): TSC33;
Begin
  //
End;
//******************* TReplication.GET_PL04_NEXTINDEX *************************

Function TReplication.GET_PL04_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
  String;
Var
  BTR_FILE          : TBTR;
  R                 : Integer;
  S                 : String;
Begin
  BTR_FILE := TBTR.Create(Application);
  Try
    BTR_FILE.FileName := BTRFILENAME;
    BTR_FILE.IgnoreSmall := True;
    Try
      BTR_FILE.Active := True;
      If BTR_FILE.RecordCount = 0 Then
        R := 0
      Else
        Begin
          BTR_FILE.First;
          S := Copy(String(BTR_FILE.DATABUFFER), POZ + 1, LEN);
          R := STRTONR(S);
          Inc(R);
        End;
      BTR_FILE.Active := False;
    Except
      R := 0;
    End;
  Finally
    BTR_FILE.Free;
  End;
  Result := FormatFloat(StringOfChar('0', LEN), R);
End;
//******************* TReplication.GET_PL05_NEXTINDEX *************************

Function TReplication.GET_PL05_NEXTINDEX(BTRFILENAME: String; POZ, LEN: Integer):
  String;
Var
  BTR_FILE          : TBTR;
  R                 : Integer;
  S                 : String;
Begin
  BTR_FILE := TBTR.Create(Application);
  Try
    BTR_FILE.FileName := BTRFILENAME;
    BTR_FILE.IgnoreSmall := True;
    Try
      BTR_FILE.Active := True;
      If BTR_FILE.RecordCount = 0 Then
        R := 0
      Else
        Begin
          BTR_FILE.First;
          S := Copy(String(BTR_FILE.DATABUFFER), POZ + 1, LEN);
          R := STRTONR(S);
          Inc(R);
        End;
      BTR_FILE.Active := False;
    Except
      R := 0;
    End;
  Finally
    BTR_FILE.Free;
  End;
  Result := FormatFloat(StringOfChar('0', LEN), R);
End;
//******************* TReplication.GET_PL21_NEXTINDEX *************************

Function TReplication.GET_PL21_NEXTINDEX(BTRFILENAME, PL21001, PL21002, PL21003: String;
  POZ1, LEN1, POZ2, LEN2, POZ3, LEN3: Integer): String;
Var
  BTR_FILE          : TBTR;
  R                 : Integer;
  S                 : String;
  KEYB              : CHARARRAY;
Begin
  R := 1;
  BTR_FILE := TBTR.Create(Application);
  Try
    BTR_FILE.FileName := BTRFILENAME;
    BTR_FILE.IgnoreSmall := True;
    BTR_FILE.KeyNum := 0;
    Try
      BTR_FILE.Active := True;
      If BTR_FILE.RecordCount = 0 Then
        R := 1
      Else
        Begin
          BTR_FILE.First;
          StrPCopy(KEYB, Format('%-' + IntToStr(LEN1) + 's' + '%-' +
            IntToStr(LEN2) + 's' + '%-' + IntToStr(LEN3) + 's', [PL21001,
            PL21002, PL21003]));
          StrCopy(BTR_FILE.KeyBuffer, KEYB);
          Try
            BTR_FILE.Equal;
            While (trimright(Copy(String(BTR_FILE.DATABUFFER), POZ1 + 1,
              LEN1)) = PL21001) And (trimright(Copy(String(
              BTR_FILE.DATABUFFER), POZ2 + 1, LEN2)) = PL21002) Do
              Begin
                S := trimright(Copy(String(BTR_FILE.DATABUFFER), POZ3 +
                  1, LEN3));
                R := STRTONR(S);
                Try
                  BTR_FILE.Next;
                Except
                  Break;
                End;
              End;
            Inc(R);
          Except
            R := 1;
          End;
        End;
      BTR_FILE.Active := False;
    Except
      R := 1;
    End;
  Finally
    BTR_FILE.Free;
  End;
  Result := FormatFloat(StringOfChar('0', LEN3), R);
End;
//******************* TReplication.GetBtrRecCount *************************

Function TReplication.GetBtrRecCount(FileName: String): Longint;
Var
  F                 : TBtr;
Begin
  F := TBtr.Create(Self);
  Try
    F.FileName := FileName;
    Try
      F.Open;
      Result := F.RecordCount;
      F.Close;
    Except
      Result := -1;
    End;
  Finally
    F.Free;
  End;
End;

Procedure Register;
Begin
  RegisterComponents('EM-Quicksoft', [TReplication]);
End;

End.

